<?php
if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

include_spip('codes_postaux_config');

function formulaires_importer_codes_postaux_charger() {

	$emplacement = sous_repertoire(_DIR_TMP, 'codes_postaux');

	$pattern = '/.*/';
	$fichier_present =  find_all_in_path($emplacement, $pattern);
	$fichier_present = array_keys($fichier_present);
	$tFichiers = codes_postaux_config_tab_fichier();
	$tFichiers = array_filter($tFichiers, fn($dom) => in_array($dom['nom_fichier'], $fichier_present));

	$valeurs = [
		'option_truncate'        => '',
		'option_relier_communes' => '',
		'nom_fichier'            => '',
		'option_filtre'          => '',
		'supprimer_fichier'      => '',
		'tFichiers'              => $tFichiers
	];


	return $valeurs;
}


function formulaires_importer_codes_postaux_verifier_dist() {
	$tFichiers = codes_postaux_config_tab_fichier();
	$tFichiers = array_column($tFichiers, null, 'slug');
	include_spip('inc/config');
	$erreurs = [];
	$emplacement = sous_repertoire(_DIR_TMP, 'codes_postaux');
	$slug = _request('nom_fichier') ?? '';
	$fichier_a_importer = $tFichiers[$slug]['nom_fichier'] ?? '';
	if (empty($fichier_a_importer)) {
		$erreurs['nom_fichier'] = _T('codes_postaux:fichier_introuvable') . " " . $emplacement . $fichier_a_importer;
	}

	if (!file_exists($emplacement . $fichier_a_importer)) {
		$erreurs['nom_fichier'] = _T('codes_postaux:fichier_introuvable') . " " . $emplacement . $fichier_a_importer;
		$erreurs['message_erreur'] = _T('codes_postaux:fichier_introuvable');
	}
	return $erreurs;
}

function formulaires_importer_codes_postaux_traiter_dist() {
	include_spip('codes_postaux_config');
	$options = [
		'truncate'        => _request("option_truncate"),
		'relier_communes' => _request("option_relier_communes"),
		'filtre'          => _request("option_filtre")
	];
	$tFichiers = codes_postaux_config_tab_fichier();
	$tFichiers = array_column($tFichiers, null, 'slug');
	$slug = _request('nom_fichier') ?? '';
	$message = codes_postaux_import($tFichiers[$slug], $options);
	if (_request('supprimer_fichier')) {
		$emplacement = sous_repertoire(_DIR_TMP, 'codes_postaux');
		$fichier_sup = $emplacement . $tFichiers[$slug]['nom_fichier'];
		unlink($fichier_sup);
	}

	return ['message_ok' => $message];
}


function codes_postaux_import($tab_fichier, $options) {
	$tab_colonne = codes_postaux_config_correspondance_colonne();
	$colonnes = $tab_colonne['colonnes'];

	if (isset($tab_colonne['filtre'])) {
		$filtre_config = $tab_colonne['filtre'];
	}
	if (isset($tab_colonne['liaison'])) {
		$liaison = $tab_colonne['liaison'];
	}


	$tab_filtres = array();
	$option_truncate = false;
	if (isset($options['truncate']))
		$option_truncate = $options['truncate'];
	$option_relier_communes = false;
	if (isset($options['relier_communes']))
		$option_relier_communes = $options['relier_communes'];
	$option_filtre = '';
	if (isset($options['filtre']))
		$option_filtre = $options['filtre'];

	$filtres = explode(';', $option_filtre);

	foreach ($filtres as $filtre) {
		$tab_temp = explode('=', $filtre);
		if (!empty($tab_filtres)) {
			$tab_filtres[$tab_temp[0]] = $tab_temp[1];
		}
	}
	$nom_fichier = $tab_fichier['nom_fichier'];
	$emplacement = sous_repertoire(_DIR_TMP, 'codes_postaux');
	$message = 'Importation du fichier ' . $nom_fichier . "<br />";
	$fichier_modele = $emplacement . $nom_fichier;


	$table = 'spip_codes_postaux';
	if ($option_truncate) {
		$message .= 'Purge de la table ' . $table . "<br />";
		spip_mysql_query('truncate table ' . $table);
		if ($option_relier_communes) {
			sql_delete("spip_cog_communes_liens", 'objet=\'code_postal\'');
		}
	}


	$pointeur_fichier = fopen($fichier_modele, "r");
	if ($pointeur_fichier <> 0) {
		$nb_ligne = 0;
		while (!feof($pointeur_fichier)) {
			$ligne = fgets($pointeur_fichier, 4096);
			$tab = explode("\t", $ligne);

			if (count($tab) > 1) {

				$tab = array_combine($tab_fichier['description'], $tab);
				if (isset($filtre_config)) {
					if (!preg_match('/^' . $filtre['valeur'] . '$/', $tab[$filtre['cle']]))
						continue;
				}
				if (!empty($tab_filtres)) {
					if (!codes_postaux_applique_filtre($tab, $tab_filtres))
						continue;
				}

				foreach ($colonnes as $nom_colonne) {
					$champs['' . $nom_colonne] = $tab[$nom_colonne];
				}

				$champs['code_pays'] = $tab_fichier['code_pays'] ?? '';

				if ($option_truncate) {
					$id_code_postal = 0;
				} else {
					$where = array();
					foreach ($champs as $key => $valeur)
						$where[] = $key . '=' . sql_quote($valeur);
					$id_code_postal = sql_getfetsel('id_code_postal', $table, implode(' AND ', $where));
				}

				if (!$id_code_postal) {
					$id_code_postal = sql_insertq($table, $champs);
				}

				if ($option_relier_communes) {
					$id_cog_commune = sql_getfetsel('id_cog_commune', 'spip_cog_communes', 'departement=' . $tab['code_departement'] . ' and (nom_majuscule = ' . sql_quote(str_replace('’', "'", strtoupper($tab['titre'])))
						. ' or concat(MID(article_majuscule,2,LENGTH(article_majuscule)-2),concat(\' \',nom_majuscule)) = ' . sql_quote(str_replace('’', "'", strtoupper($tab['titre'])))
						. ' or concat(MID(article_majuscule,2,LENGTH(article_majuscule)-2),concat(\'\',nom_majuscule)) = ' . sql_quote(str_replace('’', "'", strtoupper($tab['titre'])))
						. ')');
					if ($id_cog_commune) {
						sql_insertq("spip_cog_communes_liens", array('id_cog_commune' => $id_cog_commune, 'objet' => 'code_postal', 'id_objet' => $id_code_postal));
					}
				}
				$nb_ligne++;

			}
		}
	}
	$message .= $nb_ligne . ' enregistrements ajoutés.';
	fclose($pointeur_fichier);
	return $message;

}


function codes_postaux_applique_filtre($tab_value, $tab_filtres) {

	foreach ($tab_filtres as $col => $filtre) {
		if (isset($tab_value[$col])) {
			if (!preg_match('/^' . $filtre . '$/', $tab_value[$col])) {
				return false;
			}
		}
	}
	return true;
}
